#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@文件名称: configure_nginx.py
@功能描述: 为Django项目配置Nginx（添加配置而非替换）
@参数说明: 无
@依赖环境: Python >= 3.6
@使用方法: sudo python configure_nginx.py
"""
import os
import subprocess
import sys
from pathlib import Path
import importlib.util
import shutil
import time

# 颜色常量
COLOR_RED = "\033[31m"
COLOR_GREEN = "\033[32m"
COLOR_YELLOW = "\033[33m"
COLOR_BLUE = "\033[34m"
COLOR_RESET = "\033[0m"

def find_project_root():
    """查找包含 manage.py 的项目根目录"""
    current_dir = Path.cwd()
    while True:
        if (current_dir / "manage.py").exists():
            return current_dir
        parent_dir = current_dir.parent
        if parent_dir == current_dir:
            print(f"{COLOR_RED}错误: 未找到 manage.py{COLOR_RESET}")
            sys.exit(1)
        current_dir = parent_dir

def get_project_module(project_root):
    """获取项目模块名称（同名文件夹）"""
    project_name = project_root.name
    module_dir = project_root / project_name
    if not (module_dir / "settings.py").exists():
        print(f"{COLOR_RED}错误: 未找到 {project_name}/settings.py{COLOR_RESET}")
        sys.exit(1)
    return project_name

def get_static_root(project_root):
    """从 settings 中获取 STATIC_ROOT"""
    project_module = get_project_module(project_root)
    sys.path.insert(0, str(project_root))
    for settings_module in [f"{project_module}.settings", f"{project_module}.local_settings"]:
        spec = importlib.util.find_spec(settings_module)
        if spec:
            settings = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(settings)
            if hasattr(settings, "STATIC_ROOT"):
                static_root = Path(getattr(settings, "STATIC_ROOT")).resolve().as_posix()
                return static_root if static_root.endswith('/') else f"{static_root}/"
    print(f"{COLOR_RED}错误: 未找到 STATIC_ROOT 配置{COLOR_RESET}")
    sys.exit(1)

def generate_nginx_project_config(project_name, port, static_root):
    """生成项目特定的Nginx配置"""
    return f"""
server {{
    listen       80;
    listen       [::]:80;
    server_name  _;
    
    # 设置较大的客户端最大请求体大小，避免上传大文件时出现400错误
    client_max_body_size 20M;
    
    location /static/ {{
        alias   {static_root};
    }}

    location / {{
        uwsgi_pass  127.0.0.1:{port};
        include     uwsgi_params;
        
        # 增加uwsgi超时设置，避免长时间请求导致400错误
        uwsgi_read_timeout 300s;
        uwsgi_send_timeout 300s;
        
        # 设置请求缓冲区大小
        uwsgi_buffer_size 128k;
        uwsgi_buffers 4 256k;
        uwsgi_busy_buffers_size 256k;
    }}
}}
"""

def check_nginx_installed():
    """检查Nginx是否已安装"""
    try:
        subprocess.run(["nginx", "-v"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        return True
    except FileNotFoundError:
        return False

def ensure_nginx_running():
    """确保Nginx正在运行"""
    for _ in range(3):  # 尝试3次
        try:
            # 检查nginx是否正在运行
            status = subprocess.run("systemctl is-active nginx", shell=True, capture_output=True, text=True)
            if status.stdout.strip() == "active":
                print(f"{COLOR_GREEN}✔ Nginx 服务正在运行{COLOR_RESET}")
                return True
            
            # 尝试启动nginx
            print(f"{COLOR_YELLOW}Nginx 未运行，尝试启动...{COLOR_RESET}")
            subprocess.run("systemctl start nginx", shell=True, check=True)
            time.sleep(2)  # 等待启动
        except subprocess.CalledProcessError:
            try:
                # 尝试使用service命令（适用于旧版系统）
                subprocess.run("service nginx start", shell=True, check=True)
                time.sleep(2)  # 等待启动
            except subprocess.CalledProcessError:
                print(f"{COLOR_RED}警告: 无法通过systemctl/service启动Nginx{COLOR_RESET}")
                try:
                    # 直接启动nginx
                    subprocess.run("nginx", shell=True, check=True)
                    time.sleep(2)  # 等待启动
                except subprocess.CalledProcessError:
                    print(f"{COLOR_RED}警告: 无法启动Nginx，尝试重新启动{COLOR_RESET}")
    
    print(f"{COLOR_RED}错误: 无法确保Nginx服务运行{COLOR_RESET}")
    return False

def configure_nginx(project_name, port, static_root):
    """配置Nginx"""
    # 检查Nginx是否已安装，如果没有则安装
    if not check_nginx_installed():
        print(f"{COLOR_BLUE}正在安装Nginx...{COLOR_RESET}")
        try:
            subprocess.run("yum install -y nginx", shell=True, check=True)
        except subprocess.CalledProcessError:
            try:
                # 尝试用apt安装（适用于Debian/Ubuntu）
                subprocess.run("apt-get update && apt-get install -y nginx", shell=True, check=True)
            except subprocess.CalledProcessError:
                print(f"{COLOR_RED}错误: 无法安装Nginx，请手动安装{COLOR_RESET}")
                sys.exit(1)
    
    # 确保conf.d目录存在
    conf_dir = "/etc/nginx/conf.d"
    os.makedirs(conf_dir, exist_ok=True)
    
    # 备份现有配置（如果存在）
    project_conf_path = os.path.join(conf_dir, f"{project_name}.conf")
    if os.path.exists(project_conf_path):
        backup_path = f"{project_conf_path}.bak"
        shutil.copy2(project_conf_path, backup_path)
        print(f"{COLOR_GREEN}✔ 已备份原始配置到 {backup_path}{COLOR_RESET}")
    
    # 生成项目特定的配置文件
    config = generate_nginx_project_config(project_name, port, static_root)
    
    try:
        with open(project_conf_path, 'w') as f:
            f.write(config.strip())
        print(f"{COLOR_GREEN}✔ 项目配置已写入 {project_conf_path}{COLOR_RESET}")
        
        # 确保主配置中包含conf.d目录
        main_conf_path = "/etc/nginx/nginx.conf"
        include_line = "include /etc/nginx/conf.d/*.conf;"
        
        if os.path.exists(main_conf_path):
            with open(main_conf_path, 'r') as f:
                main_conf = f.read()
            
            # 检查是否已包含conf.d配置
            if "include /etc/nginx/conf.d/" not in main_conf:
                # 备份主配置
                shutil.copy2(main_conf_path, f"{main_conf_path}.bak")
                print(f"{COLOR_GREEN}✔ 已备份主配置到 {main_conf_path}.bak{COLOR_RESET}")
                
                # 在http块中添加include语句
                main_conf = main_conf.replace(
                    "http {", 
                    "http {\n    " + include_line
                )
                with open(main_conf_path, 'w') as f:
                    f.write(main_conf)
                print(f"{COLOR_GREEN}✔ 已在主配置中添加 conf.d 包含{COLOR_RESET}")
        else:
            print(f"{COLOR_RED}警告: 未找到主配置文件 {main_conf_path}{COLOR_RESET}")
        
        # 测试配置
        print(f"{COLOR_BLUE}正在测试Nginx配置...{COLOR_RESET}")
        test_result = subprocess.run("nginx -t", shell=True, capture_output=True, text=True)
        if test_result.returncode != 0:
            print(f"{COLOR_RED}错误: Nginx 配置测试失败{COLOR_RESET}")
            print(f"错误信息: {test_result.stderr}")
            # 尝试恢复备份
            if os.path.exists(backup_path):
                shutil.copy2(backup_path, project_conf_path)
                print(f"{COLOR_GREEN}✔ 已恢复项目配置{COLOR_RESET}")
            sys.exit(1)
        print(f"{COLOR_GREEN}✔ Nginx 配置测试通过{COLOR_RESET}")
        
        # 确保Nginx正在运行
        ensure_nginx_running()
        
        # 重启/重新加载 Nginx
        print(f"{COLOR_BLUE}正在重启Nginx...{COLOR_RESET}")
        restart_success = False
        
        # 尝试不同的重启方式
        restart_methods = [
            "systemctl restart nginx",
            "service nginx restart",
            "nginx -s reload"
        ]
        
        for method in restart_methods:
            try:
                result = subprocess.run(method, shell=True, capture_output=True, text=True)
                if result.returncode == 0:
                    print(f"{COLOR_GREEN}✔ Nginx 服务已重启 (使用 {method}){COLOR_RESET}")
                    restart_success = True
                    break
                else:
                    print(f"{COLOR_YELLOW}使用 {method} 重启失败: {result.stderr}{COLOR_RESET}")
            except Exception as e:
                print(f"{COLOR_YELLOW}使用 {method} 重启出错: {str(e)}{COLOR_RESET}")
        
        if not restart_success:
            print(f"{COLOR_RED}警告: 所有重启方法都失败，请手动重启Nginx{COLOR_RESET}")
            print("可以尝试运行: sudo nginx -s reload")
        
        # 验证Nginx是否正常运行
        time.sleep(2)  # 等待Nginx重启完成
        status_check = subprocess.run("curl -s -o /dev/null -w '%{http_code}' http://localhost/", 
                                     shell=True, capture_output=True, text=True)
        status_code = status_check.stdout.strip()
        
        if status_code and status_code not in ["000", "7"]:  # 000表示连接失败，7表示无法连接
            print(f"{COLOR_GREEN}✔ Nginx 响应状态码: {status_code}{COLOR_RESET}")
        else:
            print(f"{COLOR_YELLOW}警告: 无法连接到Nginx (状态码: {status_code}){COLOR_RESET}")
            print("这可能是因为Nginx未运行或者配置问题。")
            print("请尝试检查Nginx状态: sudo systemctl status nginx")
        
    except Exception as e:
        print(f"{COLOR_RED}错误: 配置失败 - {str(e)}{COLOR_RESET}")
        sys.exit(1)

def main():
    if os.getuid() != 0:
        print(f"{COLOR_RED}错误: 请使用 root 用户运行 (sudo python {sys.argv[0]}){COLOR_RESET}")
        sys.exit(1)

    print(f"{COLOR_BLUE}=== Django 项目 Nginx 配置工具 ==={COLOR_RESET}")
    
    project_root = find_project_root()
    project_name = project_root.name
    print(f"{COLOR_GREEN}✔ 项目名称: {project_name}{COLOR_RESET}")
    print(f"{COLOR_GREEN}✔ 项目根目录: {project_root}{COLOR_RESET}")
    
    static_root = get_static_root(project_root)
    print(f"{COLOR_GREEN}✔ 静态文件目录: {static_root}{COLOR_RESET}")
    
    port = input(f"{COLOR_BLUE}请输入 Django 运行端口(默认 8000): {COLOR_RESET}") or "8000"
    
    configure_nginx(project_name, port, static_root)
    
    print(f"\n{COLOR_GREEN}Nginx配置完成!{COLOR_RESET}")
    print(f"项目名称: {project_name}")
    print(f"静态文件映射: /static/ => {static_root}")
    print(f"Django 运行端口: {port}")
    print(f"\n访问地址: http://服务器IP/")
    print(f"\n如果遇到Bad Request (400)错误，可能的解决方法:")
    print("1. 确保Django服务在正确的端口上运行")
    print("2. 检查uwsgi配置是否正确")
    print("3. 检查请求体大小是否超过限制")
    print("4. 尝试增加超时设置")
    print("5. 检查uWSGI日志获取更多信息")

if __name__ == "__main__":
    main()